@@ -10,6 +10,7 @@ import socket
 import ctypes
 import os
 import ipaddress
+import salt.ext.six as six
 
 
 class sockaddr(ctypes.Structure):
@@ -36,7 +37,7 @@ def inet_pton(address_family, ip_string):
     # This will catch IP Addresses such as 10.1.2
     if address_family == socket.AF_INET:
         try:
-            ipaddress.ip_address(ip_string.decode())
+            ipaddress.ip_address(six.u(ip_string))
         except ValueError:
             raise socket.error('illegal IP address string passed to inet_pton')
         return socket.inet_aton(ip_string)
@@ -55,7 +56,7 @@ def inet_pton(address_family, ip_string):
     addr_size = ctypes.c_int(ctypes.sizeof(addr))
 
     if WSAStringToAddressA(
-            ip_string,
+            ip_string.encode('ascii'),
             address_family,
             None,
             ctypes.byref(addr),
