@@ -196,14 +196,14 @@ def stop(dev=None):
 
     '''
     if dev is not None:
-        log.warn('Stopping {0}, device will only reappear after reregistering!'.format(dev))
+        log.warning('Stopping {0}, device will only reappear after reregistering!'.format(dev))
         if not _bcsys(dev, 'stop', 'goaway', 'error', 'Error stopping {0}'.format(dev)):
             return False
         return _wait(lambda: _sysfs_attr(_bcpath(dev)) is False, 'error', 'Device {0} did not stop'.format(dev), 300)
     else:
         cache = uuid()
         if not cache:
-            log.warn('bcache already stopped?')
+            log.warning('bcache already stopped?')
             return None
 
         if not _alltrue(detach()):
@@ -401,7 +401,7 @@ def config_(dev=None, **kwargs):
             if key in data:
                 del data[key]
 
-        for key in data.keys():
+        for key in data:
             result.update(data[key])
 
         return result
@@ -441,7 +441,7 @@ def status(stats=False, config=False, internals=False, superblock=False, alldevs
     cdev = _bdev()
     if cdev:
         count = 0
-        for dev in statii.keys():
+        for dev in statii:
             if dev != cdev:
                 # it's a backing dev
                 if statii[dev]['cache'] == cuuid:
@@ -519,7 +519,7 @@ def device(dev, stats=False, config=False, internals=False, superblock=False):
     if 'stats' in result:
         replre = r'(stats|cache)_'
         statres = result['stats']
-        for attr in result['stats'].keys():
+        for attr in result['stats']:
             if '/' not in attr:
                 key = re.sub(replre, '', attr)
                 statres[key] = statres.pop(attr)
@@ -537,7 +537,7 @@ def device(dev, stats=False, config=False, internals=False, superblock=False):
         interres = result.pop('inter_ro', {})
         interres.update(result.pop('inter_rw', {}))
         if len(interres):
-            for key in interres.keys():
+            for key in interres:
                 if key.startswith('internal'):
                     nkey = re.sub(r'internal[s/]*', '', key)
                     interres[nkey] = interres.pop(key)
@@ -552,7 +552,7 @@ def device(dev, stats=False, config=False, internals=False, superblock=False):
     # ---------------- Config ----------------
     if config:
         configres = result['config']
-        for key in configres.keys():
+        for key in configres:
             if key.startswith('writeback'):
                 mkey, skey = re.split(r'_', key, maxsplit=1)
                 if mkey not in configres:
@@ -767,7 +767,7 @@ def _sysfs_parse(path, base_attr=None, stats=False, config=False, internals=Fals
             bintf[key].append(intf)
 
     if base_attr is not None:
-        for intf in bintf.keys():
+        for intf in bintf:
             bintf[intf] = [sintf for sintf in bintf[intf] if sintf not in base_attr]
         bintf['base'] = base_attr
 
@@ -897,10 +897,10 @@ def _wipe(dev):
         log.error('Unable to read SysFS props for {0}'.format(dev))
         return None
     elif not discard:
-        log.warn('{0} seems unable to discard'.format(dev))
+        log.warning('{0} seems unable to discard'.format(dev))
         wiper = 'dd'
     elif not HAS_BLKDISCARD:
-        log.warn('blkdiscard binary not available, properly wipe the dev manually for optimal results')
+        log.warning('blkdiscard binary not available, properly wipe the dev manually for optimal results')
         wiper = 'dd'
     else:
         wiper = 'blkdiscard'
@@ -927,7 +927,7 @@ def _wipe(dev):
 def _wait(lfunc, log_lvl=None, log_msg=None, tries=10):
     '''
     Wait for lfunc to be True
-    :return: True if lfunc succeeded within tries, False if it didnt
+    :return: True if lfunc succeeded within tries, False if it didn't
     '''
     i = 0
     while i < tries:
