@@ -4,6 +4,7 @@ Management of NTP servers on Windows
 
 .. versionadded:: 2014.1.0
 '''
+from __future__ import absolute_import
 
 # Import python libs
 import logging
@@ -22,7 +23,7 @@ def __virtual__():
     This only supports Windows
     '''
     if not salt.utils.is_windows():
-        return False
+        return (False, "Module win_system: module only works on Windows systems")
     return __virtualname__
 
 
@@ -47,9 +48,10 @@ def set_servers(*servers):
     update_cmd = ['W32tm', '/config', '/update']
 
     for cmd in server_cmd, reliable_cmd, update_cmd:
-        ret = __salt__['cmd.run'](cmd, python_shell=False)
-        if 'command completed successfully' not in ret:
-            return False
+        __salt__['cmd.run'](cmd, python_shell=False)
+
+    if not sorted(list(servers)) == get_servers():
+        return False
 
     __salt__['service.restart'](service_name)
     return True
@@ -68,7 +70,10 @@ def get_servers():
     cmd = ['w32tm', '/query', '/configuration']
     lines = __salt__['cmd.run'](cmd, python_shell=False).splitlines()
     for line in lines:
-        if line.startswith('NtpServer:'):
-            _, ntpsvrs = line.rstrip(' (Local)').split(':', 1)
-            return sorted(ntpsvrs.split())
+        try:
+            if line.startswith('NtpServer:'):
+                _, ntpsvrs = line.rsplit(' (', 1)[0].split(':', 1)
+                return sorted(ntpsvrs.split())
+        except ValueError as e:
+            return False
     return False
