@@ -3,7 +3,12 @@
 Module for managing windows systems.
 
 :depends:
+    - pythoncom
+    - pywintypes
+    - win32api
+    - win32con
     - win32net
+    - wmi
 
 Support for reboot, shutdown, etc
 '''
@@ -12,6 +17,7 @@ from __future__ import absolute_import
 # Import python libs
 import logging
 import time
+import ctypes
 from datetime import datetime
 
 # Import 3rd Party Libs
@@ -22,7 +28,6 @@ try:
     import win32api
     import win32con
     import pywintypes
-    import ctypes
     from ctypes import windll
     HAS_WIN32NET_MODS = True
 except ImportError:
@@ -32,6 +37,7 @@ except ImportError:
 import salt.utils
 import salt.utils.locales
 import salt.ext.six as six
+from salt.exceptions import CommandExecutionError
 
 # Set up logging
 log = logging.getLogger(__name__)
@@ -42,11 +48,15 @@ __virtualname__ = 'system'
 
 def __virtual__():
     '''
-    Set the system module of the kernel is Windows
+    Only works on Windows Systems with Win32 Modules
     '''
-    if HAS_WIN32NET_MODS and salt.utils.is_windows():
-        return __virtualname__
-    return (False, "Module win_system: module only works on Windows systems")
+    if not salt.utils.is_windows():
+        return False, 'Module win_system: Requires Windows'
+
+    if not HAS_WIN32NET_MODS:
+        return False, 'Module win_system: Missing win32 modules'
+
+    return __virtualname__
 
 
 def _convert_minutes_seconds(timeout, in_seconds=False):
@@ -65,6 +75,18 @@ def _convert_date_time_string(dt_string):
     return dt_obj.strftime('%Y-%m-%d %H:%M:%S')
 
 
+def _to_unicode(instr):
+    '''
+    Converts from current users character encoding to unicode.
+    When instr has a value of None, the return value of the function
+    will also be None.
+    '''
+    if instr is None or isinstance(instr, six.text_type):
+        return instr
+    else:
+        return six.text_type(instr, 'utf8')
+
+
 def halt(timeout=5, in_seconds=False):
     '''
     Halt a running system.
@@ -74,11 +96,10 @@ def halt(timeout=5, in_seconds=False):
         timeout (int):
             Number of seconds before halting the system. Default is 5 seconds.
 
-
         in_seconds (bool):
             Whether to treat timeout as seconds or minutes.
 
-        .. versionadded:: 2015.8.0
+            .. versionadded:: 2015.8.0
 
     Returns:
         bool: ``True`` if successful, otherwise ``False``
@@ -167,7 +188,7 @@ def reboot(timeout=5, in_seconds=False, wait_for_reboot=False,  # pylint: disabl
             .. versionadded:: 2015.8.0
 
         only_on_pending_reboot (bool):
-            If this is set to ``True``, then then the reboot will only proceed
+            If this is set to ``True``, then the reboot will only proceed
             if the system reports a pending reboot. Setting this parameter to
             ``True`` could be useful when calling this function from a final
             housekeeping state intended to be executed at the end of a state run
@@ -247,6 +268,11 @@ def shutdown(message=None, timeout=5, force_close=True, reboot=False,  # pylint:
             ``True`` restarts the computer immediately after shutdown. ``False``
             powers down the system. Default is ``False``.
 
+        only_on_pending_reboot (bool): If this is set to True, then the shutdown
+            will only proceed if the system reports a pending reboot. To
+            optionally shutdown in a highstate, consider using the shutdown
+            state instead of this module.
+
         only_on_pending_reboot (bool):
             If ``True`` the shutdown will only proceed if there is a reboot
             pending. ``False`` will shutdown the system. Default is ``False``.
@@ -262,6 +288,9 @@ def shutdown(message=None, timeout=5, force_close=True, reboot=False,  # pylint:
 
         salt '*' system.shutdown "System will shutdown in 5 minutes"
     '''
+    if six.PY2:
+        message = _to_unicode(message)
+
     timeout = _convert_minutes_seconds(timeout, in_seconds)
 
     if only_on_pending_reboot and not get_pending_reboot():
@@ -301,7 +330,8 @@ def shutdown_hard():
 def shutdown_abort():
     '''
     Abort a shutdown. Only available while the dialog box is being
-    displayed to the user. Once the shutdown has initiated, it cannot be aborted
+    displayed to the user. Once the shutdown has initiated, it cannot be
+    aborted.
 
     Returns:
         bool: ``True`` if successful, otherwise ``False``
@@ -360,16 +390,17 @@ def set_computer_name(name):
 
         salt 'minion-id' system.set_computer_name 'DavesComputer'
     '''
-    if name and six.PY2:
-        name = name.decode('utf-8')
+    if six.PY2:
+        name = _to_unicode(name)
 
-    if windll.kernel32.SetComputerNameExW(win32con.ComputerNamePhysicalDnsHostname,
-                                          name):
+    if windll.kernel32.SetComputerNameExW(
+            win32con.ComputerNamePhysicalDnsHostname, name):
         ret = {'Computer Name': {'Current': get_computer_name()}}
         pending = get_pending_computer_name()
         if pending not in (None, False):
             ret['Computer Name']['Pending'] = pending
         return ret
+
     return False
 
 
@@ -407,7 +438,7 @@ def get_computer_name():
     Get the Windows computer name
 
     Returns:
-        str: Returns the computer name if found. Otherwise returns ``False``
+        str: Returns the computer name if found. Otherwise returns ``False``.
 
     CLI Example:
 
@@ -437,6 +468,9 @@ def set_computer_desc(desc=None):
 
         salt 'minion-id' system.set_computer_desc 'This computer belongs to Dave!'
     '''
+    if six.PY2:
+        desc = _to_unicode(desc)
+
     # Make sure the system exists
     # Return an object containing current information array for the computer
     system_info = win32net.NetServerGetInfo(None, 101)
@@ -444,8 +478,7 @@ def set_computer_desc(desc=None):
     # If desc is passed, decode it for unicode
     if desc is None:
         return False
-    if not isinstance(desc, str):
-        desc = desc.decode('utf-8')
+
     system_info['comment'] = desc
 
     # Apply new settings
@@ -525,7 +558,7 @@ def get_computer_desc():
 
     Returns:
         str: Returns the computer description if found. Otherwise returns
-        ``False``
+        ``False``.
 
     CLI Example:
 
@@ -638,6 +671,12 @@ def join_domain(domain,
                          account_ou='ou=clients,ou=org,dc=domain,dc=tld' \\
                          account_exists=False, restart=True
     '''
+    if six.PY2:
+        domain = _to_unicode(domain)
+        username = _to_unicode(username)
+        password = _to_unicode(password)
+        account_ou = _to_unicode(account_ou)
+
     status = get_domain_workgroup()
     if 'Domain' in status:
         if status['Domain'] == domain:
@@ -654,6 +693,56 @@ def join_domain(domain,
         account_ou = account_ou.split('\\')
         account_ou = ''.join(account_ou)
 
+    err = _join_domain(domain=domain, username=username, password=password,
+                       account_ou=account_ou, account_exists=account_exists)
+
+    if not err:
+        ret = {'Domain': domain,
+               'Restart': False}
+        if restart:
+            ret['Restart'] = reboot()
+        return ret
+
+    raise CommandExecutionError(win32api.FormatMessage(err).rstrip())
+
+
+def _join_domain(domain,
+                 username=None,
+                 password=None,
+                 account_ou=None,
+                 account_exists=False):
+    '''
+    Helper function to join the domain.
+
+    Args:
+        domain (str): The domain to which the computer should be joined, e.g.
+            ``example.com``
+
+        username (str): Username of an account which is authorized to join
+            computers to the specified domain. Need to be either fully qualified
+            like ``user@domain.tld`` or simply ``user``
+
+        password (str): Password of the specified user
+
+        account_ou (str): The DN of the OU below which the account for this
+            computer should be created when joining the domain, e.g.
+            ``ou=computers,ou=departm_432,dc=my-company,dc=com``
+
+        account_exists (bool): If set to ``True`` the computer will only join
+            the domain if the account already exists. If set to ``False`` the
+            computer account will be created if it does not exist, otherwise it
+            will use the existing account. Default is False.
+
+    Returns:
+        int:
+
+    :param domain:
+    :param username:
+    :param password:
+    :param account_ou:
+    :param account_exists:
+    :return:
+    '''
     NETSETUP_JOIN_DOMAIN = 0x1  # pylint: disable=invalid-name
     NETSETUP_ACCOUNT_CREATE = 0x2  # pylint: disable=invalid-name
     NETSETUP_DOMAIN_JOIN_IF_JOINED = 0x20  # pylint: disable=invalid-name
@@ -669,23 +758,13 @@ def join_domain(domain,
     pythoncom.CoInitialize()
     conn = wmi.WMI()
     comp = conn.Win32_ComputerSystem()[0]
-    err = comp.JoinDomainOrWorkgroup(Name=domain,
-                                     Password=password,
-                                     UserName=username,
-                                     AccountOU=account_ou,
-                                     FJoinOptions=join_options)
 
-    # you have to do this because JoinDomainOrWorkgroup returns a strangely
-    # formatted value that looks like (0,)
-    if not err[0]:
-        ret = {'Domain': domain,
-               'Restart': False}
-        if restart:
-            ret['Restart'] = reboot()
-        return ret
-
-    log.error(win32api.FormatMessage(err[0]).rstrip())
-    return False
+    # Return the results of the command as an error
+    # JoinDomainOrWorkgroup returns a strangely formatted value that looks like
+    # (0,) so return the first item
+    return comp.JoinDomainOrWorkgroup(
+        Name=domain, Password=password, UserName=username, AccountOU=account_ou,
+        FJoinOptions=join_options)[0]
 
 
 def unjoin_domain(username=None,
@@ -741,6 +820,11 @@ def unjoin_domain(username=None,
                          password='unjoinpassword' disable=True \\
                          restart=True
     '''
+    if six.PY2:
+        username = _to_unicode(username)
+        password = _to_unicode(password)
+        domain = _to_unicode(domain)
+
     status = get_domain_workgroup()
     if 'Workgroup' in status:
         if status['Workgroup'] == workgroup:
@@ -755,7 +839,7 @@ def unjoin_domain(username=None,
     if username and password is None:
         return 'Must specify a password if you pass a username'
 
-    NETSETUP_ACCT_DELETE = 0x2  # pylint: disable=invalid-name
+    NETSETUP_ACCT_DELETE = 0x4  # pylint: disable=invalid-name
 
     unjoin_options = 0x0
     if disable:
@@ -804,7 +888,6 @@ def get_domain_workgroup():
     .. code-block:: bash
 
         salt 'minion-id' system.get_domain_workgroup
-
     '''
     pythoncom.CoInitialize()
     conn = wmi.WMI()
@@ -872,6 +955,7 @@ def set_system_time(newtime):
 
         newtime (str):
             The time to set. Can be any of the following formats:
+
             - HH:MM:SS AM/PM
             - HH:MM AM/PM
             - HH:MM:SS (24 hour)
@@ -973,12 +1057,15 @@ def set_system_date_time(years=None,
         system_time.wSecond = int(seconds)
         system_time_ptr = ctypes.pointer(system_time)
         succeeded = ctypes.windll.kernel32.SetLocalTime(system_time_ptr)
-        return succeeded is not 0
-    except OSError:
+        if succeeded is not 0:
+            return True
+        else:
+            log.error('Failed to set local time')
+            raise CommandExecutionError(
+                win32api.FormatMessage(succeeded).rstrip())
+    except OSError as err:
         log.error('Failed to set local time')
-        return False
-
-    return True
+        raise CommandExecutionError(err)
 
 
 def get_system_date():
@@ -1005,6 +1092,7 @@ def set_system_date(newdate):
     Args:
         newdate (str):
             The date to set. Can be any of the following formats
+
             - YYYY-MM-DD
             - MM-DD-YYYY
             - MM-DD-YY
@@ -1197,7 +1285,7 @@ def get_pending_servermanager():
     key = r'SOFTWARE\Microsoft\ServerManager'
 
     # There are situations where it's possible to have '(value not set)' as
-    # the value data, and since an actual reboot wont be pending in that
+    # the value data, and since an actual reboot won't be pending in that
     # instance, just catch instances where we try unsuccessfully to cast as int.
 
     reg_ret = __salt__['reg.read_value']('HKLM', key, vname)
@@ -1276,7 +1364,6 @@ def set_reboot_required_witnessed():
     .. code-block:: bash
 
         salt '*' system.set_reboot_required_witnessed
-
     '''
     return __salt__['reg.set_value'](hive='HKLM',
                                      key=MINION_VOLATILE_KEY,
