@@ -138,8 +138,8 @@ renderer will take care of fetching the file for you, parsing it with all of
 the pyobjects features available and then place the requested objects in the
 global scope of the template being rendered.
 
-This works for both types of import statements, ``import X`` and
-``from X import Y``.
+This works for all types of import statements; ``import X``,
+``from X import Y``, and ``from X import Y as Z``.
 
 .. code-block:: python
    :linenos:
@@ -148,18 +148,17 @@ This works for both types of import statements, ``import X`` and
 
     import salt://myfile.sls
     from salt://something/data.sls import Object
+    from salt://something/data.sls import Object as Other
 
 
 See the Map Data section for a more practical use.
 
 Caveats:
 
-* You cannot use the ``as`` syntax, you can only import objects using their
-  existing name.
-
 * Imported objects are ALWAYS put into the global scope of your template,
   regardless of where your import statement is.
 
+
 Salt object
 ^^^^^^^^^^^
 In the spirit of the object interface for creating state data pyobjects also
@@ -224,21 +223,57 @@ different grain matches.
 
     class Samba(Map):
         merge = 'samba:lookup'
+        # NOTE: priority is new to 2017.7.0
+        priority = ('os_family', 'os')
+
+        class Ubuntu:
+            __grain__ = 'os'
+            service = 'smbd'
 
         class Debian:
             server = 'samba'
             client = 'samba-client'
             service = 'samba'
 
-        class Ubuntu:
-            __grain__ = 'os'
-            service = 'smbd'
-
-        class RedHat:
+        class RHEL:
+            __match__ = 'RedHat'
             server = 'samba'
             client = 'samba'
             service = 'smb'
 
+.. note::
+    By default, the ``os_family`` grain will be used as the target for
+    matching. This can be overridden by specifying a ``__grain__`` attribute.
+
+    If a ``__match__`` attribute is defined for a given class, then that value
+    will be matched against the targeted grain, otherwise the class name's
+    value will be be matched.
+
+    Given the above example, the following is true:
+
+    1. Minions with an ``os_family`` of **Debian** will be assigned the
+       attributes defined in the **Debian** class.
+    2. Minions with an ``os`` grain of **Ubuntu** will be assigned the
+       attributes defined in the **Ubuntu** class.
+    3. Minions with an ``os_family`` grain of **RedHat** will be assigned the
+       attributes defined in the **RHEL** class.
+
+    That said, sometimes a minion may match more than one class. For instance,
+    in the above example, Ubuntu minions will match both the **Debian** and
+    **Ubuntu** classes, since Ubuntu has an ``os_family`` grain of **Debian**
+    an an ``os`` grain of **Ubuntu**. As of the 2017.7.0 release, the order is
+    dictated by the order of declaration, with classes defined later overriding
+    earlier ones. Additionally, 2017.7.0 adds support for explicitly defining
+    the ordering using an optional attribute called ``priority``.
+
+    Given the above example, ``os_family`` matches will be processed first,
+    with ``os`` matches processed after. This would have the effect of
+    assigning ``smbd`` as the ``service`` attribute on Ubuntu minions. If the
+    ``priority`` item was not defined, or if the order of the items in the
+    ``priority`` tuple were reversed, Ubuntu minions would have a ``service``
+    attribute of ``samba``, since ``os_family`` matches would have been
+    processed second.
+
 To use this new data you can import it into your state file and then access
 your attributes. To access the data in the map you simply access the attribute
 name on the base class that is extending Map. Assuming the above Map was in the
@@ -259,22 +294,24 @@ TODO
 * Interface for working with reactor files
 '''
 
+# Import Python Libs
 from __future__ import absolute_import
-
 import logging
+import os
 import re
-from salt.ext.six import exec_
 
+# Import Salt Libs
+from salt.ext.six import exec_
 import salt.utils
 import salt.loader
-
 from salt.fileclient import get_file_client
 from salt.utils.pyobjects import Registry, StateFactory, SaltObject, Map
+import salt.ext.six as six
 
 # our import regexes
-FROM_RE = r'^\s*from\s+(salt:\/\/.*)\s+import (.*)$'
-IMPORT_RE = r'^\s*import\s+(salt:\/\/.*)$'
-FROM_AS_RE = r'^(.*) as (.*)$'
+FROM_RE = re.compile(r'^\s*from\s+(salt:\/\/.*)\s+import (.*)$')
+IMPORT_RE = re.compile(r'^\s*import\s+(salt:\/\/.*)$')
+FROM_AS_RE = re.compile(r'^(.*) as (.*)$')
 
 log = logging.getLogger(__name__)
 
@@ -284,6 +321,17 @@ except NameError:
     __context__ = {}
 
 
+class PyobjectsModule(object):
+    '''This provides a wrapper for bare imports.'''
+
+    def __init__(self, name, attrs):
+        self.name = name
+        self.__dict__ = attrs
+
+    def __repr__(self):
+        return "<module '{0!s}' (pyobjects)>".format(self.name)
+
+
 def load_states():
     '''
     This loads our states into the salt __context__
@@ -293,11 +341,18 @@ def load_states():
     # the loader expects to find pillar & grain data
     __opts__['grains'] = salt.loader.grains(__opts__)
     __opts__['pillar'] = __pillar__
-    lazy_funcs = salt.loader.minion_mods(__opts__)
-    lazy_states = salt.loader.states(__opts__, lazy_funcs)
+    lazy_utils = salt.loader.utils(__opts__)
+    lazy_funcs = salt.loader.minion_mods(__opts__, utils=lazy_utils)
+    lazy_serializers = salt.loader.serializers(__opts__)
+    lazy_states = salt.loader.states(__opts__,
+            lazy_funcs,
+            lazy_utils,
+            lazy_serializers)
 
     # TODO: some way to lazily do this? This requires loading *all* state modules
-    for key, func in lazy_states.iteritems():
+    for key, func in six.iteritems(lazy_states):
+        if '.' not in key:
+            continue
         mod_name, func_name = key.split('.', 1)
         if mod_name not in states:
             states[mod_name] = {}
@@ -374,59 +429,73 @@ def render(template, saltenv='base', sls='', salt_data=True, **kwargs):
     # so that they may bring in objects from other files. while we do this we
     # disable the registry since all we're looking for here is python objects,
     # not salt state data
-    template_data = []
     Registry.enabled = False
-    for line in template.readlines():
-        line = line.rstrip('\r\n')
-        matched = False
-        for RE in (IMPORT_RE, FROM_RE):
-            matches = re.match(RE, line)
-            if not matches:
-                continue
 
-            import_file = matches.group(1).strip()
-            try:
-                imports = matches.group(2).split(',')
-            except IndexError:
-                # if we don't have a third group in the matches object it means
-                # that we're importing everything
-                imports = None
+    def process_template(template):
+        template_data = []
+        # Do not pass our globals to the modules we are including and keep the root _globals untouched
+        template_globals = dict(_globals)
+        for line in template.readlines():
+            line = line.rstrip('\r\n')
+            matched = False
+            for RE in (IMPORT_RE, FROM_RE):
+                matches = RE.match(line)
+                if not matches:
+                    continue
 
-            state_file = client.cache_file(import_file, saltenv)
-            if not state_file:
-                raise ImportError("Could not find the file {0!r}".format(import_file))
+                import_file = matches.group(1).strip()
+                try:
+                    imports = matches.group(2).split(',')
+                except IndexError:
+                    # if we don't have a third group in the matches object it means
+                    # that we're importing everything
+                    imports = None
 
-            with salt.utils.fopen(state_file) as f:
-                state_contents = f.read()
+                state_file = client.cache_file(import_file, saltenv)
+                if not state_file:
+                    raise ImportError(
+                        'Could not find the file \'{0}\''.format(import_file)
+                    )
 
-            state_locals = {}
-            exec_(state_contents, _globals, state_locals)
+                with salt.utils.fopen(state_file) as state_fh:
+                    state_contents, state_globals = process_template(state_fh)
+                exec_(state_contents, state_globals)
 
-            if imports is None:
-                imports = list(state_locals.keys())
+                # if no imports have been specified then we are being imported as: import salt://foo.sls
+                # so we want to stick all of the locals from our state file into the template globals
+                # under the name of the module -> i.e. foo.MapClass
+                if imports is None:
+                    import_name = os.path.splitext(os.path.basename(state_file))[0]
+                    template_globals[import_name] = PyobjectsModule(import_name, state_globals)
+                else:
+                    for name in imports:
+                        name = alias = name.strip()
 
-            for name in imports:
-                name = alias = name.strip()
+                        matches = FROM_AS_RE.match(name)
+                        if matches is not None:
+                            name = matches.group(1).strip()
+                            alias = matches.group(2).strip()
 
-                matches = re.match(FROM_AS_RE, name)
-                if matches is not None:
-                    name = matches.group(1).strip()
-                    alias = matches.group(2).strip()
+                        if name not in state_globals:
+                            raise ImportError(
+                                '\'{0}\' was not found in \'{1}\''.format(
+                                    name,
+                                    import_file
+                                )
+                            )
+                        template_globals[alias] = state_globals[name]
 
-                if name not in state_locals:
-                    raise ImportError("{0!r} was not found in {1!r}".format(
-                        name,
-                        import_file
-                    ))
-                _globals[alias] = state_locals[name]
+                matched = True
+                break
 
-            matched = True
-            break
+            if not matched:
+                template_data.append(line)
 
-        if not matched:
-            template_data.append(line)
+        return "\n".join(template_data), template_globals
 
-    final_template = "\n".join(template_data)
+    # process the template that triggered the render
+    final_template, final_globals = process_template(template)
+    _globals.update(final_globals)
 
     # re-enable the registry
     Registry.enabled = True
