@@ -7,8 +7,15 @@ Set up the version of Salt
 from __future__ import absolute_import, print_function
 import re
 import sys
+import locale
 import platform
 
+# linux_distribution deprecated in py3.7
+try:
+    from platform import linux_distribution
+except ImportError:
+    from distro import linux_distribution
+
 # pylint: disable=invalid-name,redefined-builtin
 # Import 3rd-party libs
 from salt.ext import six
@@ -87,12 +94,12 @@ class SaltStackVersion(object):
         'Beryllium'     : (2015, 8),
         'Boron'         : (2016, 3),
         'Carbon'        : (2016, 11),
-        'Nitrogen'      : (MAX_SIZE - 102, 0),
+        'Nitrogen'      : (2017, 7),
         'Oxygen'        : (MAX_SIZE - 101, 0),
         'Fluorine'      : (MAX_SIZE - 100, 0),
-        'Neon'          : (MAX_SIZE - 99, 0),
+        'Neon'          :  (MAX_SIZE - 99, 0),
+        'Sodium'        :  (MAX_SIZE - 98, 0),
         # pylint: disable=E8265
-        #'Sodium'       : (MAX_SIZE - 98 , 0),
         #'Magnesium'    : (MAX_SIZE - 97 , 0),
         #'Aluminium'    : (MAX_SIZE - 96 , 0),
         #'Silicon'      : (MAX_SIZE - 95 , 0),
@@ -505,6 +512,9 @@ def __discover_version(saltstack_version):
             process = subprocess.Popen(
                 ['git', 'describe', '--tags', '--match', 'v[0-9]*', '--always'], **kwargs)
             out, err = process.communicate()
+        if six.PY3:
+            out = out.decode()
+            err = err.decode()
         out = out.strip()
         err = err.strip()
 
@@ -627,7 +637,7 @@ def system_information():
         '''
         Return host system version.
         '''
-        lin_ver = platform.linux_distribution()
+        lin_ver = linux_distribution()
         mac_ver = platform.mac_ver()
         win_ver = platform.win32_ver()
 
@@ -646,7 +656,7 @@ def system_information():
     version = system_version()
     release = platform.release()
     if platform.win32_ver()[0]:
-        import win32api
+        import win32api  # pylint: disable=3rd-party-module-not-gated
         server = {'Vista': '2008Server',
                   '7': '2008ServerR2',
                   '8': '2012Server',
@@ -666,10 +676,11 @@ def system_information():
 
     system = [
         ('system', platform.system()),
-        ('dist', ' '.join(platform.dist())),
+        ('dist', ' '.join(linux_distribution(full_distribution_name=False))),
         ('release', release),
         ('machine', platform.machine()),
         ('version', version),
+        ('locale', locale.getpreferredencoding()),
     ]
 
     for name, attr in system:
@@ -718,13 +729,13 @@ def versions_report(include_salt_cloud=False):
 
 def msi_conformant_version():
     '''
-    A msi conformant version consists of up to 4 numbers, each smaller than 256,
-    except the 4th. Therefore, the year must be represented as 'short year'.
+    An msi conformant version consists of up to 4 numbers, each smaller than 256, except the 4th.
+    Therefore, the year must be represented as 'short year'.
 
     Examples (depend on git checkout):
       develop                2016.11.0-742-g5ca4d20     16.11.0.742
-      2016.11 (branch)       2016.11.2-78-gce1f01f      16.11.2.78
-      v2016.11.2 (tag)       2016.11.2                  16.11.2.0
+      20166.11 (branch)      2016.11.2-78-gce1f01f      16.11.2.78
+      v20166.11.2 (tag)      2016.11.2                  16.11.2.0
 
     Note that the commit count for tags is 0(zero)
     '''
