@@ -5,37 +5,51 @@ from __future__ import absolute_import
 from __future__ import print_function
 import json
 import time
-from distutils.version import StrictVersion  # pylint: disable=import-error,no-name-in-module
+import threading
 
 # Import Salt Libs
+import salt.utils
 from salt.netapi.rest_tornado import saltnado
-from unit.netapi.rest_tornado.test_handlers import SaltnadoTestCase
+from salt.utils.versions import StrictVersion
 
 # Import Salt Testing Libs
-from salttesting import skipIf
-from salttesting.helpers import ensure_in_syspath
+from tests.unit.netapi.rest_tornado.test_handlers import SaltnadoTestCase
+from tests.support.helpers import flaky
+from tests.support.unit import skipIf
 
-ensure_in_syspath('../../../')
+# Import 3rd-party libs
+from salt.ext import six
+from salt.utils.zeromq import zmq, ZMQDefaultLoop as ZMQIOLoop
+HAS_ZMQ_IOLOOP = bool(zmq)
 
-import salt.ext.six as six
 
-try:
-    import zmq
-    from zmq.eventloop.ioloop import ZMQIOLoop
-    HAS_ZMQ_IOLOOP = True
-except ImportError:
-    HAS_ZMQ_IOLOOP = False
+def json_loads(data):
+    if six.PY3 and isinstance(data, bytes):
+        data = data.decode('utf-8')
+    return json.loads(data)
+
+
+class _SaltnadoIntegrationTestCase(SaltnadoTestCase):  # pylint: disable=abstract-method
+
+    @property
+    def opts(self):
+        return self.get_config('client_config', from_scratch=True)
+
+    @property
+    def mod_opts(self):
+        return self.get_config('minion', from_scratch=True)
 
 
 @skipIf(HAS_ZMQ_IOLOOP is False, 'PyZMQ version must be >= 14.0.1 to run these tests.')
 @skipIf(StrictVersion(zmq.__version__) < StrictVersion('14.0.1'), 'PyZMQ must be >= 14.0.1 to run these tests.')
-class TestSaltAPIHandler(SaltnadoTestCase):
+class TestSaltAPIHandler(_SaltnadoIntegrationTestCase):
     def get_app(self):
         urls = [('/', saltnado.SaltAPIHandler)]
 
         application = self.build_tornado_app(urls)
 
         application.event_listener = saltnado.EventListener({}, self.opts)
+        self.application = application
         return application
 
     def test_root(self):
@@ -47,13 +61,9 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                               request_timeout=30,
                 )
         self.assertEqual(response.code, 200)
-        response_obj = json.loads(response.body)
-        self.assertItemsEqual(response_obj['clients'],
-                              ['runner',
-                               'runner_async',
-                               'local_async',
-                               'local']
-                              )
+        response_obj = json_loads(response.body)
+        self.assertEqual(sorted(response_obj['clients']),
+                         ['local', 'local_async', 'runner', 'runner_async'])
         self.assertEqual(response_obj['return'], 'Welcome')
 
     def test_post_no_auth(self):
@@ -76,8 +86,6 @@ class TestSaltAPIHandler(SaltnadoTestCase):
         self.assertEqual(response.code, 302)
         self.assertEqual(response.headers['Location'], '/login')
 
-    # Local client tests
-    @skipIf(True, 'to be re-enabled when #23623 is merged')
     def test_simple_local_post(self):
         '''
         Test a basic API of /
@@ -94,8 +102,9 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                               connect_timeout=30,
                               request_timeout=30,
                               )
-        response_obj = json.loads(response.body)
-        self.assertEqual(response_obj['return'], [{'minion': True, 'sub_minion': True}])
+        response_obj = json_loads(response.body)
+        self.assertEqual(len(response_obj['return']), 1)
+        self.assertEqual(response_obj['return'][0], {'minion': True, 'sub_minion': True})
 
     def test_simple_local_post_no_tgt(self):
         '''
@@ -113,11 +122,9 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                               connect_timeout=30,
                               request_timeout=30,
                               )
-        response_obj = json.loads(response.body)
+        response_obj = json_loads(response.body)
         self.assertEqual(response_obj['return'], ["No minions matched the target. No command was sent, no jid was assigned."])
 
-    # local client request body test
-    @skipIf(True, 'Undetermined race condition in test. Temporarily disabled.')
     def test_simple_local_post_only_dictionary_request(self):
         '''
         Test a basic API of /
@@ -134,8 +141,9 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                               connect_timeout=30,
                               request_timeout=30,
                               )
-        response_obj = json.loads(response.body)
-        self.assertEqual(response_obj['return'], [{'minion': True, 'sub_minion': True}])
+        response_obj = json_loads(response.body)
+        self.assertEqual(len(response_obj['return']), 1)
+        self.assertEqual(response_obj['return'][0], {'minion': True, 'sub_minion': True})
 
     def test_simple_local_post_invalid_request(self):
         '''
@@ -165,7 +173,7 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               )
 
-        response_obj = json.loads(response.body)
+        response_obj = json_loads(response.body)
         ret = response_obj['return']
         ret[0]['minions'] = sorted(ret[0]['minions'])
 
@@ -190,7 +198,7 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               )
 
-        response_obj = json.loads(response.body)
+        response_obj = json_loads(response.body)
         ret = response_obj['return']
         ret[0]['minions'] = sorted(ret[0]['minions'])
         ret[1]['minions'] = sorted(ret[1]['minions'])
@@ -224,7 +232,7 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               )
 
-        response_obj = json.loads(response.body)
+        response_obj = json_loads(response.body)
         ret = response_obj['return']
         ret[0]['minions'] = sorted(ret[0]['minions'])
         ret[1]['minions'] = sorted(ret[1]['minions'])
@@ -247,9 +255,31 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               )
-        response_obj = json.loads(response.body)
+        response_obj = json_loads(response.body)
         self.assertEqual(response_obj['return'], [{}])
 
+    def test_simple_local_post_only_dictionary_request_with_order_masters(self):
+        '''
+        Test a basic API of /
+        '''
+        low = {'client': 'local',
+                'tgt': '*',
+                'fun': 'test.ping',
+              }
+        response = self.fetch('/',
+                              method='POST',
+                              body=salt.utils.json.dumps(low),
+                              headers={'Content-Type': self.content_type_map['json'],
+                                       saltnado.AUTH_TOKEN_HEADER: self.token['token']},
+                              connect_timeout=30,
+                              request_timeout=30,
+                              )
+        response_obj = salt.utils.json.loads(response.body)
+
+        self.application.opts['order_masters'] = []
+        self.application.opts['syndic_wait'] = 5
+        self.assertEqual(response_obj['return'], [{'minion': True, 'sub_minion': True}])
+
     # runner tests
     def test_simple_local_runner_post(self):
         low = [{'client': 'runner',
@@ -263,7 +293,7 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                               connect_timeout=30,
                               request_timeout=30,
                               )
-        response_obj = json.loads(response.body)
+        response_obj = json_loads(response.body)
         self.assertEqual(len(response_obj['return']), 1)
         self.assertEqual(set(response_obj['return'][0]), set(['minion', 'sub_minion']))
 
@@ -280,15 +310,16 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                               connect_timeout=10,
                               request_timeout=10,
                               )
-        response_obj = json.loads(response.body)
+        response_obj = json_loads(response.body)
         self.assertIn('return', response_obj)
         self.assertEqual(1, len(response_obj['return']))
         self.assertIn('jid', response_obj['return'][0])
         self.assertIn('tag', response_obj['return'][0])
 
 
+@flaky
 @skipIf(HAS_ZMQ_IOLOOP is False, 'PyZMQ version must be >= 14.0.1 to run these tests.')
-class TestMinionSaltAPIHandler(SaltnadoTestCase):
+class TestMinionSaltAPIHandler(_SaltnadoIntegrationTestCase):
     def get_app(self):
         urls = [(r"/minions/(.*)", saltnado.MinionSaltAPIHandler),
                 (r"/minions", saltnado.MinionSaltAPIHandler),
@@ -304,7 +335,7 @@ class TestMinionSaltAPIHandler(SaltnadoTestCase):
                               headers={saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               follow_redirects=False,
                               )
-        response_obj = json.loads(response.body)
+        response_obj = json_loads(response.body)
         self.assertEqual(len(response_obj['return']), 1)
         # one per minion
         self.assertEqual(len(response_obj['return'][0]), 2)
@@ -319,7 +350,7 @@ class TestMinionSaltAPIHandler(SaltnadoTestCase):
                               headers={saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               follow_redirects=False,
                               )
-        response_obj = json.loads(response.body)
+        response_obj = json_loads(response.body)
         self.assertEqual(len(response_obj['return']), 1)
         self.assertEqual(len(response_obj['return'][0]), 1)
         # check a single grain
@@ -336,7 +367,7 @@ class TestMinionSaltAPIHandler(SaltnadoTestCase):
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               )
 
-        response_obj = json.loads(response.body)
+        response_obj = json_loads(response.body)
         ret = response_obj['return']
         ret[0]['minions'] = sorted(ret[0]['minions'])
 
@@ -358,7 +389,7 @@ class TestMinionSaltAPIHandler(SaltnadoTestCase):
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               )
 
-        response_obj = json.loads(response.body)
+        response_obj = json_loads(response.body)
         ret = response_obj['return']
         ret[0]['minions'] = sorted(ret[0]['minions'])
 
@@ -387,7 +418,7 @@ class TestMinionSaltAPIHandler(SaltnadoTestCase):
 
 
 @skipIf(HAS_ZMQ_IOLOOP is False, 'PyZMQ version must be >= 14.0.1 to run these tests.')
-class TestJobsSaltAPIHandler(SaltnadoTestCase):
+class TestJobsSaltAPIHandler(_SaltnadoIntegrationTestCase):
     def get_app(self):
         urls = [(r"/jobs/(.*)", saltnado.JobsSaltAPIHandler),
                 (r"/jobs", saltnado.JobsSaltAPIHandler),
@@ -406,7 +437,7 @@ class TestJobsSaltAPIHandler(SaltnadoTestCase):
                                follow_redirects=False,
                                )
         response = self.wait(timeout=30)
-        response_obj = json.loads(response.body)['return'][0]
+        response_obj = json_loads(response.body)['return'][0]
         try:
             for jid, ret in six.iteritems(response_obj):
                 self.assertIn('Function', ret)
@@ -416,7 +447,7 @@ class TestJobsSaltAPIHandler(SaltnadoTestCase):
                 self.assertIn('StartTime', ret)
                 self.assertIn('Arguments', ret)
         except AttributeError as attribute_error:
-            print(json.loads(response.body))
+            print(json_loads(response.body))
             raise
 
         # test with a specific JID passed in
@@ -428,7 +459,7 @@ class TestJobsSaltAPIHandler(SaltnadoTestCase):
                                follow_redirects=False,
                                )
         response = self.wait(timeout=30)
-        response_obj = json.loads(response.body)['return'][0]
+        response_obj = json_loads(response.body)['return'][0]
         self.assertIn('Function', response_obj)
         self.assertIn('Target', response_obj)
         self.assertIn('Target-type', response_obj)
@@ -441,7 +472,7 @@ class TestJobsSaltAPIHandler(SaltnadoTestCase):
 # TODO: run all the same tests from the root handler, but for now since they are
 # the same code, we'll just sanity check
 @skipIf(HAS_ZMQ_IOLOOP is False, 'PyZMQ version must be >= 14.0.1 to run these tests.')
-class TestRunSaltAPIHandler(SaltnadoTestCase):
+class TestRunSaltAPIHandler(_SaltnadoIntegrationTestCase):
     def get_app(self):
         urls = [("/run", saltnado.RunSaltAPIHandler),
                 ]
@@ -461,12 +492,12 @@ class TestRunSaltAPIHandler(SaltnadoTestCase):
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               )
-        response_obj = json.loads(response.body)
+        response_obj = json_loads(response.body)
         self.assertEqual(response_obj['return'], [{'minion': True, 'sub_minion': True}])
 
 
 @skipIf(HAS_ZMQ_IOLOOP is False, 'PyZMQ version must be >= 14.0.1 to run these tests.')
-class TestEventsSaltAPIHandler(SaltnadoTestCase):
+class TestEventsSaltAPIHandler(_SaltnadoIntegrationTestCase):
     def get_app(self):
         urls = [(r"/events", saltnado.EventsSaltAPIHandler),
                 ]
@@ -489,6 +520,8 @@ class TestEventsSaltAPIHandler(SaltnadoTestCase):
         self.stop()
 
     def on_event(self, event):
+        if six.PY3:
+            event = event.decode('utf-8')
         if self.events_to_fire > 0:
             self.application.event_listener.event.fire_event({
                 'foo': 'bar',
@@ -510,7 +543,7 @@ class TestEventsSaltAPIHandler(SaltnadoTestCase):
 
 
 @skipIf(HAS_ZMQ_IOLOOP is False, 'PyZMQ version must be >= 14.0.1 to run these tests.')
-class TestWebhookSaltAPIHandler(SaltnadoTestCase):
+class TestWebhookSaltAPIHandler(_SaltnadoIntegrationTestCase):
 
     def get_app(self):
 
@@ -526,35 +559,40 @@ class TestWebhookSaltAPIHandler(SaltnadoTestCase):
 
     @skipIf(True, 'Skipping until we can devote more resources to debugging this test.')
     def test_post(self):
-        def verify_event(future):
-            '''
-            Verify that the event fired on the master matches what we sent
-            '''
-            event = future.result()
+        self._future_resolved = threading.Event()
+        try:
+            def verify_event(future):
+                '''
+                Notify the threading event that the future is resolved
+                '''
+                self._future_resolved.set()
+
+            self._finished = False  # TODO: remove after some cleanup of the event listener
+
+            # get an event future
+            future = self.application.event_listener.get_event(self,
+                                                               tag='salt/netapi/hook',
+                                                               callback=verify_event)
+            # fire the event
+            response = self.fetch('/hook',
+                                  method='POST',
+                                  body='foo=bar',
+                                  headers={saltnado.AUTH_TOKEN_HEADER: self.token['token']},
+                                  )
+            response_obj = json_loads(response.body)
+            self.assertTrue(response_obj['success'])
+            resolve_future_timeout = 60
+            self._future_resolved.wait(resolve_future_timeout)
+            try:
+                event = future.result()
+            except Exception as exc:
+                self.fail('Failed to resolve future under {} secs: {}'.format(resolve_future_timeout, exc))
             self.assertEqual(event['tag'], 'salt/netapi/hook')
             self.assertIn('headers', event['data'])
-            self.assertEqual(event['data']['post'], {'foo': 'bar'})
-        # get an event future
-        self._finished = False  # TODO: remove after some cleanup of the event listener
-        event = self.application.event_listener.get_event(self,
-                                                          tag='salt/netapi/hook',
-                                                          callback=verify_event,
-                                                          )
-        # fire the event
-        response = self.fetch('/hook',
-                              method='POST',
-                              body='foo=bar',
-                              headers={saltnado.AUTH_TOKEN_HEADER: self.token['token']},
-                              )
-        response_obj = json.loads(response.body)
-        self.assertTrue(response_obj['success'])
-
-
-if __name__ == '__main__':
-    from integration import run_tests  # pylint: disable=import-error
-    run_tests(TestEventsSaltAPIHandler,
-              TestJobsSaltAPIHandler,
-              TestMinionSaltAPIHandler,
-              TestRunSaltAPIHandler,
-              TestSaltAPIHandler,
-              TestWebhookSaltAPIHandler, needs_daemon=True)
+            self.assertEqual(
+                event['data']['post'],
+                {'foo': salt.utils.to_bytes('bar')}
+            )
+        finally:
+            self._future_resolved.clear()
+            del self._future_resolved
