@@ -33,7 +33,7 @@ class NpmStateTest(ModuleCase, SaltReturnAssertsMixin):
         Basic test to determine if NPM module was successfully installed and
         removed.
         '''
-        ret = self.run_state('npm.installed', name='pm2')
+        ret = self.run_state('npm.installed', name='pm2', registry="http://registry.npmjs.org/")
         self.assertSaltTrueReturn(ret)
         ret = self.run_state('npm.removed', name='pm2')
         self.assertSaltTrueReturn(ret)
@@ -44,14 +44,23 @@ class NpmStateTest(ModuleCase, SaltReturnAssertsMixin):
         '''
         Determine if URL-referenced NPM module can be successfully installed.
         '''
-        user = os.environ.get('SUDO_USER', 'root')
-        npm_dir = os.path.join(RUNTIME_VARS.TMP, 'git-install-npm')
-        self.run_state('file.directory', name=npm_dir, user=user, dir_mode='755')
-        ret = self.run_state('npm.installed', name='request/request#v2.81.1', runas=user, dir=npm_dir)
+        if LooseVersion(cmd.run('npm -v')) >= LooseVersion(MAX_NPM_VERSION):
+            user = os.environ.get('SUDO_USER', 'root')
+            npm_dir = os.path.join(RUNTIME_VARS.TMP, 'git-install-npm')
+            self.run_state('file.directory', name=npm_dir, user=user, dir_mode='755')
+        else:
+            user = None
+            npm_dir = None
+        ret = self.run_state('npm.installed',
+                             name='request/request#v2.81.1',
+                             runas=user,
+                             dir=npm_dir,
+                             registry="http://registry.npmjs.org/")
         self.assertSaltTrueReturn(ret)
         ret = self.run_state('npm.removed', name='git://github.com/request/request', runas=user, dir=npm_dir)
         self.assertSaltTrueReturn(ret)
-        self.run_state('file.absent', name=npm_dir)
+        if npm_dir is not None:
+            self.run_state('file.absent', name=npm_dir)
 
     @requires_network()
     @destructiveTest
@@ -60,7 +69,7 @@ class NpmStateTest(ModuleCase, SaltReturnAssertsMixin):
         Basic test to determine if NPM module successfully installs multiple
         packages.
         '''
-        ret = self.run_state('npm.installed', name=None, pkgs=['pm2', 'grunt'])
+        ret = self.run_state('npm.installed', name=None, pkgs=['pm2', 'grunt'], registry="http://registry.npmjs.org/")
         self.assertSaltTrueReturn(ret)
 
     @skipIf(salt.utils.which('npm') and LooseVersion(cmd.run('npm -v')) >= LooseVersion(MAX_NPM_VERSION),
