@@ -22,6 +22,7 @@ import time
 import stat
 import errno
 import signal
+import textwrap
 import logging
 import tempfile
 import subprocess
@@ -120,14 +121,17 @@ class ShellTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
                     data = '\n'.join(data)
                     self.assertIn('minion', data)
         '''
-        arg_str = '-c {0} {1}'.format(self.get_config_dir(), arg_str)
-        return self.run_script('salt', arg_str, with_retcode=with_retcode, catch_stderr=catch_stderr)
+        arg_str = '-c {0} -t {1} {2}'.format(self.get_config_dir(), timeout, arg_str)
+        return self.run_script('salt', arg_str, with_retcode=with_retcode, catch_stderr=catch_stderr, timeout=timeout)
 
-    def run_ssh(self, arg_str, with_retcode=False, timeout=25, catch_stderr=False):
+    def run_ssh(self, arg_str, with_retcode=False, timeout=25,
+                catch_stderr=False, wipe=False, raw=False):
         '''
         Execute salt-ssh
         '''
-        arg_str = '-c {0} -i --priv {1} --roster-file {2} localhost {3} --out=json'.format(
+        arg_str = '{0} {1} -c {2} -i --priv {3} --roster-file {4} localhost {5} --out=json'.format(
+            ' -W' if wipe else '',
+            ' -r' if raw else '',
             self.get_config_dir(),
             os.path.join(RUNTIME_VARS.TMP_CONF_DIR, 'key_test'),
             os.path.join(RUNTIME_VARS.TMP_CONF_DIR, 'roster'),
@@ -206,8 +210,10 @@ class ShellTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
         arg_str = '--config-dir {0} {1}'.format(self.get_config_dir(), arg_str)
         return self.run_script('salt-cp', arg_str, with_retcode=with_retcode, catch_stderr=catch_stderr)
 
-    def run_call(self, arg_str, with_retcode=False, catch_stderr=False):
-        arg_str = '--config-dir {0} {1}'.format(self.get_config_dir(), arg_str)
+    def run_call(self, arg_str, with_retcode=False, catch_stderr=False, local=False):
+        arg_str = '{0} --config-dir {1} {2}'.format('--local' if local else '',
+                                                    self.get_config_dir(), arg_str)
+
         return self.run_script('salt-call', arg_str, with_retcode=with_retcode, catch_stderr=catch_stderr)
 
     def run_cloud(self, arg_str, catch_stderr=False, timeout=None):
@@ -442,11 +448,24 @@ class ShellCase(ShellTestCase, AdaptedConfigurationTestCaseMixin, ScriptPathMixi
                                catch_stderr=catch_stderr,
                                timeout=timeout)
 
-    def run_ssh(self, arg_str, with_retcode=False, catch_stderr=False, timeout=60):  # pylint: disable=W0221
+    def run_spm(self, arg_str, with_retcode=False, catch_stderr=False, timeout=60):  # pylint: disable=W0221
+        '''
+        Execute spm
+        '''
+        return self.run_script('spm',
+                               arg_str,
+                               with_retcode=with_retcode,
+                               catch_stderr=catch_stderr,
+                               timeout=timeout)
+
+    def run_ssh(self, arg_str, with_retcode=False, catch_stderr=False,
+                timeout=60, wipe=True, raw=False):  # pylint: disable=W0221
         '''
         Execute salt-ssh
         '''
-        arg_str = '-ldebug -W -c {0} -i --priv {1} --roster-file {2} --out=json localhost {3}'.format(
+        arg_str = '{0} -ldebug{1} -c {2} -i --priv {3} --roster-file {4} --out=json localhost {5}'.format(
+            ' -W' if wipe else '',
+            ' -r' if raw else '',
             self.get_config_dir(),
             os.path.join(RUNTIME_VARS.TMP_CONF_DIR, 'key_test'),
             os.path.join(RUNTIME_VARS.TMP_CONF_DIR, 'roster'),
@@ -532,11 +551,12 @@ class ShellCase(ShellTestCase, AdaptedConfigurationTestCaseMixin, ScriptPathMixi
                                catch_stderr=catch_stderr,
                                timeout=60)
 
-    def run_call(self, arg_str, with_retcode=False, catch_stderr=False):
+    def run_call(self, arg_str, with_retcode=False, catch_stderr=False, local=False):
         '''
         Execute salt-call.
         '''
-        arg_str = '--config-dir {0} {1}'.format(self.get_config_dir(), arg_str)
+        arg_str = '{0} --config-dir {1} {2}'.format('--local' if local else '',
+                                                    self.get_config_dir(), arg_str)
         return self.run_script('salt-call',
                                arg_str,
                                with_retcode=with_retcode,
@@ -554,6 +574,128 @@ class ShellCase(ShellTestCase, AdaptedConfigurationTestCaseMixin, ScriptPathMixi
                                timeout=timeout)
 
 
+class SPMTestUserInterface(object):
+    '''
+    Test user interface to SPMClient
+    '''
+    def __init__(self):
+        self._status = []
+        self._confirm = []
+        self._error = []
+
+    def status(self, msg):
+        self._status.append(msg)
+
+    def confirm(self, action):
+        self._confirm.append(action)
+
+    def error(self, msg):
+        self._error.append(msg)
+
+
+class SPMCase(TestCase, AdaptedConfigurationTestCaseMixin):
+    '''
+    Class for handling spm commands
+    '''
+
+    def _spm_build_files(self, config):
+        self.formula_dir = os.path.join(' '.join(config['file_roots']['base']), 'formulas')
+        self.formula_sls_dir = os.path.join(self.formula_dir, 'apache')
+        self.formula_sls = os.path.join(self.formula_sls_dir, 'apache.sls')
+        self.formula_file = os.path.join(self.formula_dir, 'FORMULA')
+
+        dirs = [self.formula_dir, self.formula_sls_dir]
+        for f_dir in dirs:
+            os.makedirs(f_dir)
+
+        import salt.utils
+
+        with salt.utils.fopen(self.formula_sls, 'w') as fp:
+            fp.write(textwrap.dedent('''\
+                     install-apache:
+                       pkg.installed:
+                         - name: apache2
+                     '''))
+
+        with salt.utils.fopen(self.formula_file, 'w') as fp:
+            fp.write(textwrap.dedent('''\
+                     name: apache
+                     os: RedHat, Debian, Ubuntu, Suse, FreeBSD
+                     os_family: RedHat, Debian, Suse, FreeBSD
+                     version: 201506
+                     release: 2
+                     summary: Formula for installing Apache
+                     description: Formula for installing Apache
+                     '''))
+
+    def _spm_config(self, assume_yes=True):
+        self._tmp_spm = tempfile.mkdtemp()
+        config = self.get_temp_config('minion', **{
+            'spm_logfile': os.path.join(self._tmp_spm, 'log'),
+            'spm_repos_config': os.path.join(self._tmp_spm, 'etc', 'spm.repos'),
+            'spm_cache_dir': os.path.join(self._tmp_spm, 'cache'),
+            'spm_build_dir': os.path.join(self._tmp_spm, 'build'),
+            'spm_build_exclude': ['apache/.git'],
+            'spm_db_provider': 'sqlite3',
+            'spm_files_provider': 'local',
+            'spm_db': os.path.join(self._tmp_spm, 'packages.db'),
+            'extension_modules': os.path.join(self._tmp_spm, 'modules'),
+            'file_roots': {'base': [self._tmp_spm, ]},
+            'formula_path': os.path.join(self._tmp_spm, 'salt'),
+            'pillar_path': os.path.join(self._tmp_spm, 'pillar'),
+            'reactor_path': os.path.join(self._tmp_spm, 'reactor'),
+            'assume_yes': True if assume_yes else False,
+            'force': False,
+            'verbose': False,
+            'cache': 'localfs',
+            'cachedir': os.path.join(self._tmp_spm, 'cache'),
+            'spm_repo_dups': 'ignore',
+            'spm_share_dir': os.path.join(self._tmp_spm, 'share'),
+        })
+
+        import salt.utils
+        import yaml
+
+        if not os.path.isdir(config['formula_path']):
+            os.makedirs(config['formula_path'])
+
+        with salt.utils.fopen(os.path.join(self._tmp_spm, 'spm'), 'w') as fp:
+            fp.write(yaml.dump(config))
+
+        return config
+
+    def _spm_create_update_repo(self, config):
+
+        build_spm = self.run_spm('build', self.config, self.formula_dir)
+
+        c_repo = self.run_spm('create_repo', self.config,
+                              self.config['spm_build_dir'])
+
+        repo_conf_dir = self.config['spm_repos_config'] + '.d'
+        os.makedirs(repo_conf_dir)
+
+        import salt.utils
+
+        with salt.utils.fopen(os.path.join(repo_conf_dir, 'spm.repo'), 'w') as fp:
+            fp.write(textwrap.dedent('''\
+                     local_repo:
+                       url: file://{0}
+                     '''.format(self.config['spm_build_dir'])))
+
+        u_repo = self.run_spm('update_repo', self.config)
+
+    def _spm_client(self, config):
+        import salt.spm
+        self.ui = SPMTestUserInterface()
+        client = salt.spm.SPMClient(self.ui, config)
+        return client
+
+    def run_spm(self, cmd, config, arg=None):
+        client = self._spm_client(config)
+        spm_cmd = client.run([cmd, arg])
+        return self.ui._status
+
+
 class ModuleCase(TestCase, SaltClientTestCaseMixin):
     '''
     Execute a module function
@@ -674,11 +816,12 @@ class SSHCase(ShellCase):
     def _arg_str(self, function, arg):
         return '{0} {1}'.format(function, ' '.join(arg))
 
-    def run_function(self, function, arg=(), timeout=90, **kwargs):
+    def run_function(self, function, arg=(), timeout=90, wipe=True, raw=False, **kwargs):
         '''
         We use a 90s timeout here, which some slower systems do end up needing
         '''
-        ret = self.run_ssh(self._arg_str(function, arg), timeout=timeout)
+        ret = self.run_ssh(self._arg_str(function, arg), timeout=timeout,
+                           wipe=wipe, raw=raw)
         try:
             return json.loads(ret)['localhost']
         except Exception:
